## Module Logging
### Étapes pour enregistrer les logs dans un fichier :

1. **Spécifier l'emplacement du fichier de log**.
2. **Définir le mode du fichier** en fonction de l'action souhaitée :
   - `'a'` pour **ajouter** au fichier existant.
   - `'w'` pour **remplacer** le fichier existant.
3. **Configurer le format de log** pour inclure des informations comme la date et l'heure, le niveau de log, et le message.

### Exemple de code : Enregistrer les logs dans un fichier

Voici un exemple complet qui enregistre des messages de log dans un fichier avec l'option de choisir entre ajouter ou remplacer.

#### Exemple de code :

```python
import logging

# Chemin du fichier où les logs seront enregistrés
log_file = '/chemin/vers/votre/fichier.log'

# Définir le mode : 'w' pour write, 'a' pour append
file_mode = 'a'  # Changer en 'w' (write)
# si vous voulez remplacer le fichier de log à chaque lancement du script

# Configuration du logging pour sauvegarder les logs dans un fichier
logging.basicConfig(
    level=logging.DEBUG,  # Niveau de log : DEBUG, INFO, WARNING, ERROR, CRITICAL
    format='%(asctime)s - %(levelname)s - %(message)s',  # Format du log : inclut l'heure, le niveau, et le message
    datefmt='%Y-%m-%d %H:%M:%S',  # Format de la date et de l'heure
    filename=log_file,  # Spécifie le fichier de log
    filemode=file_mode  # Définit le mode d'ouverture du fichier : 'w' pour remplacer, 'a' pour ajouter
)

# Exemples de messages de log
logging.debug('Ceci est un message de débogage.')
logging.info('Ceci est un message d\'information.')
logging.warning('Ceci est un message d\'avertissement.')
logging.error('Ceci est un message d\'erreur.')
logging.critical('Ceci est un message critique.')
```

### Explication :

- **`filename=log_file`** :
  - Ce paramètre spécifie le chemin vers le fichier où les logs seront enregistrés. Par exemple, si vous mettez `'/chemin/vers/votre/fichier.log'`, ce sera l'emplacement du fichier de log.
  
- **`filemode='a'`** :
  - `'a'` (pour **ajouter**) : Si le fichier existe déjà, les nouveaux logs seront ajoutés à la suite des anciens. Aucun contenu ne sera supprimé.
  - `'w'` (pour **remplacer**) : Si le fichier existe, son contenu sera effacé chaque fois que le programme sera exécuté, et les nouveaux logs remplaceront les anciens.

### Chemin d'exemple pour le fichier :
```bash
log_file = 'logging.log'
```


### Exemple de sortie dans le fichier de log :

```bash
2024-10-10 12:45:36 - DEBUG - Ceci est un message de débogage.
2024-10-10 12:45:36 - INFO - Ceci est un message d'information.
2024-10-10 12:45:36 - WARNING - Ceci est un message d'avertissement.
2024-10-10 12:45:36 - ERROR - Ceci est un message d'erreur.
2024-10-10 12:45:36 - CRITICAL - Ceci est un message critique.
```

### Explication détaillée des paramètres :
- **`level=logging.DEBUG`** : Définit le niveau minimum de log. Ici, tous les logs à partir du niveau DEBUG seront enregistrés 
  - DEBUG (plus bas niveau) 
  - INFO, 
  - WARNING, 
  - ERROR, 
  - CRITICAL (plus haut niveau)
  
- **`format='%(asctime)s - %(levelname)s - %(message)s'`** : Spécifie le format des logs. Chaque ligne de log inclura :
  - `%asctime` : L'heure à laquelle le log a été généré.
  - `%levelname` : Le niveau de log (DEBUG, INFO, etc.).
  - `%message` : Le message de log. Possible d'afficher la valeur de variables par exemple
  
- **`datefmt='%Y-%m-%d %H:%M:%S'`** : Ce paramètre personnalise le format d'affichage de la date et de l'heure (ici, sous la forme "AAAA-MM-JJ HH:MM:SS").

| Niveau              | Valeur numérique | Description                                                                                              |
|---------------------|------------------|----------------------------------------------------------------------------------------------------------|
| `logging.NOTSET`     | 0                | Lorsqu'il est défini sur un logger, cela indique que les loggers ancêtres doivent être consultés pour déterminer le niveau effectif. Si cela résout toujours à `NOTSET`, alors tous les événements sont enregistrés. Lorsqu'il est défini sur un handler, tous les événements sont traités. |
| `logging.DEBUG`      | 10               | Informations détaillées, généralement d'intérêt uniquement pour un développeur cherchant à diagnostiquer un problème. |
| `logging.INFO`       | 20               | Confirmation que tout fonctionne comme prévu.                                                             |
| `logging.WARNING`    | 30               | Une indication qu'il s'est passé quelque chose d'inattendu, ou qu'un problème pourrait survenir prochainement (par ex. « espace disque faible »). Le logiciel fonctionne toujours comme prévu. |
| `logging.ERROR`      | 40               | En raison d'un problème plus sérieux, le logiciel n'a pas été en mesure d'exécuter une fonction.           |
| `logging.CRITICAL`   | 50               | Une erreur grave, indiquant que le programme lui-même peut ne pas être en mesure de continuer à s'exécuter. |
